from collections import OrderedDict
import logging
import os
import shutil

from . import config
from . import db
from .db import Project, Repository, Build, Test
from .utils import call, mkdir

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('action')


def do_init():
    mkdir(config.DOT_DIR)
    mkdir(config.BUILD_DIR)
    mkdir(config.TEMPLATE_DIR)
    mkdir(config.STATIC_DIR)

    # Create config file
    if config.CONFIG is None:
        empty_config = {
            'my-project': OrderedDict((
                ('name', 'My Project'),
                ('path', '.'),
                ('repository', {}),
                ('triage', {}),
                ('test', {}),
                ('build', {}),
            ))
        }
        config.save(empty_config)

    # Init db
    db.create_all()

    code_dir = os.path.dirname(os.path.realpath(__file__))
    tpl_dir = os.path.join(code_dir, 'template')
    for name in os.listdir(tpl_dir):
        dest = os.path.join(config.TEMPLATE_DIR, name)
        if os.path.exists(dest):
            continue
        src = os.path.join(tpl_dir, name)
        shutil.copy(src, dest)

    static_dir = os.path.join(code_dir, 'static')
    for name in os.listdir(static_dir):
        dest = os.path.join(config.STATIC_DIR, name)
        if os.path.exists(dest):
            continue
        src = os.path.join(static_dir, name)
        shutil.copy(src, config.STATIC_DIR)


def do_trigger():
    with db.session() as session:
        db.check_config(Project, config.CONFIG)
        query = session.query(Project).filter_by(active=True)
        for project in query:
            # Sync config
            ctx = {'project_id': project.id}
            conf = config.CONFIG.get(project.key)
            repo_conf = conf.get('repository', {})
            db.check_config(Repository, repo_conf, **ctx)

            test_conf = conf.get('test', {})
            db.check_config(Test, test_conf, **ctx)

            build_conf = conf.get('build', {})
            if not build_conf:
                continue

            # Sync repos
            project.sync_repositories()

            # Find dirty ones
            dirty = project.dirty_repositories()
            if not dirty:
                continue

            # Create build and runs
            Build.new_build(build_conf, project, dirty)


def do_build():
    while True:
        with db.session() as session:
            next_build = Build.next_waiting_build()
            if not next_build:
                break
            next_build.launch()

def do_serve():
    from .web import app
    host = config.cli('host')
    port = config.cli('port')
    app.run(host=host, port=port, debug=True)
